@@ -42,24 +42,13 @@
 #include <signal.h>
 
 #include <glib.h>
-#include <libvolume_id.h>
+#include <blkid.h>
 
 #include "libhal/libhal.h"
 #include "partutil/partutil.h"
 #include "linux_dvd_rw_utils.h"
 #include "../../logger.h"
 
-static void vid_log(int priority, const char *file, int line, const char *format, ...)
-{
-	char log_str[1024];
-	va_list args;
-
-	va_start(args, format);
-	vsnprintf(log_str, sizeof(log_str), format, args);
-	logger_forward_debug("%s:%i %s\n", file, line, log_str);
-	va_end(args);
-}
-
 static gchar *
 strdup_valid_utf8 (const char *str)
 {
@@ -89,7 +78,7 @@ strdup_valid_utf8 (const char *str)
 
 
 static void
-set_volume_id_values (LibHalContext *ctx, const char *udi, LibHalChangeSet *cs, struct volume_id *vid)
+set_blkid_values (LibHalChangeSet *cs, blkid_probe pr)
 {
 	char buf[256];
 	const char *usage;
@@ -101,28 +90,28 @@ set_volume_id_values (LibHalContext *ctx, const char *udi, LibHalChangeSet *cs,
 
 	dbus_error_init (&error);
 
-	if (!volume_id_get_usage(vid, &usage))
+	if (blkid_probe_lookup_value(pr, "USAGE", &usage, NULL))
 		usage = "";
 	libhal_changeset_set_property_string (cs, "volume.fsusage", usage);
 	HAL_DEBUG (("volume.fsusage = '%s'", usage));
 
-	if (!volume_id_get_type(vid, &type))
+	if (blkid_probe_lookup_value(pr, "TYPE", &type, NULL))
 		type = "";
-	if (!libhal_changeset_set_property_string (cs, "volume.fstype", type))
+	if (libhal_changeset_set_property_string (cs, "volume.fstype", type))
 		libhal_changeset_set_property_string (cs, "volume.fstype", "");
 	HAL_DEBUG(("volume.fstype = '%s'", type));
 
-	if (!volume_id_get_type_version(vid, &type_version))
+	if (blkid_probe_lookup_value(pr, "VERSION", &type_version, NULL))
 		type_version = "";
 	libhal_changeset_set_property_string (cs, "volume.fsversion", type_version);
 	HAL_DEBUG(("volume.fsversion = '%s'", type_version));
 
-	if (!volume_id_get_uuid(vid, &uuid))
+	if (blkid_probe_lookup_value(pr, "UUID", &uuid, NULL))
 		uuid = "";
 	libhal_changeset_set_property_string (cs, "volume.uuid", uuid);
 	HAL_DEBUG(("volume.uuid = '%s'", uuid));
 
-	if (!volume_id_get_label(vid, &label))
+	if (blkid_probe_lookup_value(pr, "LABEL", &label, NULL))
 		label = "";
 
 	if (label[0] != '\0') {
@@ -153,8 +142,7 @@ set_volume_id_values (LibHalContext *ctx, const char *udi, LibHalChangeSet *cs,
 }
 
 static void
-advanced_disc_detect (LibHalContext *ctx, const char *udi, LibHalChangeSet *cs,
-                      int fd, const char *device_file)
+advanced_disc_detect (LibHalChangeSet *cs, int fd, const char *device_file)
 {
 	/* the discs block size */
 	unsigned short bs; 
@@ -312,12 +300,12 @@ main (int argc, char *argv[])
 	LibHalContext *ctx = NULL;
 	DBusError error;
 	char *parent_udi;
-	char *sysfs_path;
-	struct volume_id *vid;
+	blkid_probe pr;
 	char *stordev_dev_file;
 	char *partition_number_str;
 	char *partition_start_str;
 	char *is_disc_str;
+	char *fsusage;
 	dbus_bool_t is_disc;
 	unsigned int partition_number;
 	guint64 partition_start;
@@ -332,9 +320,6 @@ main (int argc, char *argv[])
 	cs = NULL;
 	disc_may_have_data = FALSE;
 
-	/* hook in our debug into libvolume_id */
-	volume_id_log_fn = vid_log;
-
 	setup_logger ();
 
 	/* assume failure */
@@ -346,7 +331,7 @@ main (int argc, char *argv[])
 		goto out;
 	if ((parent_udi = getenv ("HAL_PROP_INFO_PARENT")) == NULL)
 		goto out;
-	if ((sysfs_path = getenv ("HAL_PROP_LINUX_SYSFS_PATH")) == NULL)
+	if (getenv ("HAL_PROP_LINUX_SYSFS_PATH") == NULL)
 		goto out;
 	partition_number_str = getenv ("HAL_PROP_VOLUME_PARTITION_NUMBER");
 	if (partition_number_str != NULL)
@@ -366,6 +351,8 @@ main (int argc, char *argv[])
 	else
 		is_disc = FALSE;
 
+	fsusage = getenv ("HAL_PROP_VOLUME_FSUSAGE");
+
 	dbus_error_init (&error);
 	if ((ctx = libhal_ctx_init_direct (&error)) == NULL)
 		goto out;
@@ -436,7 +423,7 @@ main (int argc, char *argv[])
 		case CDS_XA_2_2:
 			libhal_changeset_set_property_bool (cs, "volume.disc.has_data", TRUE);
 			HAL_DEBUG(("Disc in %s has data", device_file));
-			advanced_disc_detect (ctx, udi, cs, fd, device_file);
+			advanced_disc_detect (cs, fd, device_file);
 			break;
 		case CDS_NO_INFO:	/* blank or invalid CD */
 			libhal_changeset_set_property_bool (cs, "volume.disc.is_blank", TRUE);
@@ -475,8 +462,10 @@ main (int argc, char *argv[])
 			case 0x05: /* Advance Storage – Magneto-Optical */
 				libhal_changeset_set_property_string (cs, "volume.disc.type", "mo");
 				libhal_changeset_set_property_bool (cs, "volume.disc.is_rewritable", TRUE);
+				break;
 			case 0x04: /* Magneto Optical write once */
 				libhal_changeset_set_property_string (cs, "volume.disc.type", "mo");
+				break;
 			case 0x08: /* CD-ROM */
 				libhal_changeset_set_property_string (cs, "volume.disc.type", "cd_rom");
 				break;
@@ -502,6 +491,10 @@ main (int argc, char *argv[])
 				libhal_changeset_set_property_string (cs, "volume.disc.type", "dvd_rw");
 				libhal_changeset_set_property_bool (cs, "volume.disc.is_rewritable", TRUE);
 				break;
+			case 0x15: /* DVD-R Dual Layer Sequential */
+			case 0x16: /* DVD-R Dual Layer Jump */
+				libhal_changeset_set_property_string (cs, "volume.disc.type", "dvd_r_dl");
+				break;
 			case 0x1A: /* DVD+RW */
 				libhal_changeset_set_property_string (cs, "volume.disc.type", "dvd_plus_rw");
 				libhal_changeset_set_property_bool (cs, "volume.disc.is_rewritable", TRUE);
@@ -601,8 +594,15 @@ main (int argc, char *argv[])
 		}
 	}
 
+	if (fsusage != NULL && strlen(fsusage) > 0) {
+		HAL_DEBUG(("have already information about fsusage from udev, no need to probe for filesystem"));
+		should_probe_for_fs = FALSE;
+	}
+
 	if (should_probe_for_fs) {
 
+		HAL_DEBUG(("start probing for filesystem ..."));
+
 		if ((stordev_dev_file = libhal_device_get_property_string (
 			     ctx, parent_udi, "block.device", &error)) == NULL) {
 			goto out;
@@ -617,23 +617,33 @@ main (int argc, char *argv[])
 		}
 
 		/* probe for file system */
-		vid = volume_id_open_fd (fd);
-		if (vid != NULL) {
-			int vid_ret;
-			HAL_INFO (("invoking volume_id_probe_all, offset=%d, size=%d", vol_probe_offset, vol_size));
-			vid_ret = volume_id_probe_all (vid, vol_probe_offset, vol_size);
-			HAL_INFO (("volume_id_probe_all returned %d", vid_ret));
+		pr = blkid_new_probe ();
+		if (pr != NULL) {
+			int bid_ret;
 
-			if (vid_ret != 0 && is_disc && vol_probe_offset != 0) {
-				/* Some cd-rom drives report the offset of the session in the cd's TOC
-				 * wrong.  Fallback to probing at offset 0, just to be sure */
-				HAL_INFO (("invoking volume_id_probe_all, offset=0, size=%d", vol_size));
-				vid_ret = volume_id_probe_all (vid, 0 , vol_size);
-				HAL_INFO (("volume_id_probe_all returned %d", vid_ret));
+			blkid_probe_set_request (pr, BLKID_PROBREQ_LABEL | BLKID_PROBREQ_UUID |
+						 BLKID_PROBREQ_TYPE | BLKID_PROBREQ_SECTYPE |
+						 BLKID_PROBREQ_USAGE | BLKID_PROBREQ_VERSION);
+
+			HAL_INFO (("invoking blkid_do_safeprobe, offset=%d, size=%d", vol_probe_offset, vol_size));
+			bid_ret = blkid_probe_set_device (pr, fd, vol_probe_offset, vol_size);
+			if (bid_ret == 0) {
+				bid_ret = blkid_do_safeprobe (pr);
+				HAL_INFO (("blkid_do_safeprobe returned %d", bid_ret));
 			}
 
-			if (vid_ret == 0) {
-				set_volume_id_values(ctx, udi, cs, vid);
+			if (bid_ret != 0 && is_disc && vol_probe_offset != 0) {
+				/* Some cd-rom drives report the offset of the session in the cd's TOC
+				 * wrong.  Fallback to probing at offset 0, just to be sure */
+				HAL_INFO (("invoking blkid_do_safeprobe, offset=0, size=%d", vol_size));
+				bid_ret = blkid_probe_set_device (pr, fd, 0, vol_size);
+				if (bid_ret == 0)
+					bid_ret = blkid_do_safeprobe (pr);
+				HAL_INFO (("blkid_do_safeprobe returned %d", bid_ret));
+			}
+
+			if (bid_ret == 0) {
+				set_blkid_values(cs, pr);
 				if (disc_may_have_data) {
 					libhal_changeset_set_property_bool (cs, "volume.disc.is_blank", FALSE);
 					libhal_changeset_set_property_bool (cs, "volume.disc.has_data", TRUE);
@@ -649,7 +659,7 @@ main (int argc, char *argv[])
 			 *  this is good enough for now... the only discs I know of that does this
 			 *  is in fact Apple's install disc.)
 			 */
-			if (vid_ret != 0 && is_disc) {
+			if (bid_ret != 0 && is_disc) {
 				PartitionTable *p;
 				p = part_table_load_from_disk (stordev_dev_file);
 				if (p != NULL) {
@@ -667,11 +677,10 @@ main (int argc, char *argv[])
 							guint64 part_offset;
 
 							part_offset = part_table_entry_get_offset (p, i);
-							if (volume_id_probe_all (
-								    vid, vol_probe_offset + part_offset, 0) == 0) {
-
-								set_volume_id_values(ctx, udi, cs, vid);
-							}
+							if (blkid_probe_set_device (pr, fd,
+								vol_probe_offset + part_offset, 0) == 0 &&
+							    blkid_do_safeprobe (pr) == 0)
+								set_blkid_values(cs, pr);
 
 							/* and we're done */
 							break;
@@ -685,7 +694,7 @@ main (int argc, char *argv[])
 				}
 			}
 
-			volume_id_close(vid);
+			blkid_free_probe (pr);
 		}
 
 		/* get partition type number, if we find a msdos partition table */
