@@ -417,7 +417,7 @@ device_is_mounted (const char *device, char **mount_point)
 	return ret;
 }
 
-/* maps volume_id fs types to the appropriate -t mount option */
+/* maps blkid fs types to the appropriate -t mount option */
 static const char *
 map_fstype (const char *fstype)
 {
@@ -426,6 +426,8 @@ map_fstype (const char *fstype)
 		return "cd9660";
 	else if (! strcmp (fstype, "ext2"))
 		return "ext2fs";
+	else if (! strcmp (fstype, "ext3"))
+		return "ext2fs";
 	else if (! strcmp (fstype, "vfat"))
 		return "msdosfs";
 #elif sun
@@ -456,6 +458,8 @@ handle_mount (LibHalContext *hal_ctx,
 	GError *err = NULL;
 	char *sout = NULL;
 	char *serr = NULL;
+	char *mount_option_commasep = NULL;
+	char *mount_do_fstype = NULL;
 	int exit_status;
 	char *args[10];
 	int na;
@@ -465,6 +469,7 @@ handle_mount (LibHalContext *hal_ctx,
 	char *action;
 	gboolean is_remount;
 	gboolean explicit_mount_point_given;
+	gboolean found_alternative_fstype = FALSE;
 	const char *end;
 #ifdef __FreeBSD__
 	struct passwd *pw;
@@ -581,12 +586,6 @@ handle_mount (LibHalContext *hal_ctx,
 		explicit_mount_point_given = FALSE;
 		if (strlen (mount_point) == 0) {
 			char *p;
-			const char *label;
-			
-			if (volume != NULL)
-				label = libhal_volume_get_label (volume);
-			else
-				label = NULL;
 			
 			if (label != NULL) {
 				/* best - use label */
@@ -642,10 +641,76 @@ handle_mount (LibHalContext *hal_ctx,
 			
 			i++;
 		}
+	}	
+
+	/* construct arguments to mount */
+	na = 0;
+	
+	args[na++] = MOUNT;
+
+	if (strlen (mount_fstype) > 0) {
+		mount_do_fstype = (char *) map_fstype (mount_fstype);
+		if (volume && strcmp(mount_do_fstype, mount_fstype) == 0) {
+			/* there was nothing mapped and we have a volume */
+			const char *_fstype;
+
+			_fstype = libhal_volume_get_fstype (volume);
+			/* check if the given fstype differs from the volume.fstype */
+			if (_fstype && (strcmp(_fstype, mount_fstype) != 0)) {
+				/* the fstype differs from the give fstype mount options, check if it's allowed as alternative */
+				char **alternative;
+
+				dbus_error_init (&error);
+			        alternative = libhal_device_get_property_strlist (hal_ctx, udi, "volume.fstype.alternative", &error);
+			        if (dbus_error_is_set (&error)) {
+			                unknown_error ("Cannot get volume.fstype.alternative");
+			                dbus_error_free (&error);
+        			} else if (alternative != NULL) {
+					for (i = 0; alternative[i] != NULL; i++) {
+#ifdef DEBUG
+						printf ("check volume.fstype.alternative[%d]='%s' for given fstype '%s'\n", i, alternative[i], mount_fstype);
+#endif
+						if (strcmp(alternative[i], mount_fstype) == 0) {
+							printf ("found_alternative_fstype = TRUE\n");
+							found_alternative_fstype = TRUE;
+							break;
+						}
+					}
+					libhal_free_string_array(alternative);
+				}
+				if (!found_alternative_fstype) {
+					/* We have a given fstype option, which isn't allowed as 
+					   as alternative fstype.
+					   TODO: add error message and handling or exit/refuse mount
+					*/
+				}
+
+			}
+		}
+
+	} else if (volume == NULL) {
+		/* non-pollable drive; force auto */
+		mount_do_fstype = "auto";
+	} else if (libhal_volume_get_fstype (volume) != NULL && strlen (libhal_volume_get_fstype (volume)) > 0) {
+		mount_do_fstype = (char *) map_fstype (libhal_volume_get_fstype (volume));
+	} else {
+		mount_do_fstype = "auto";
 	}
 
+	/* check the mount options */
 	dbus_error_init (&error);
-	allowed_options = libhal_device_get_property_strlist (hal_ctx, udi, "volume.mount.valid_options", &error);
+	
+	if (found_alternative_fstype) {
+		char key[128];
+
+		sprintf(key, "volume.mount.%s.valid_options", mount_do_fstype);		
+#ifdef DEBUG
+		printf ("found alternative fstype='mount_do_fstype' checking now '%s'", key);
+#endif
+		allowed_options = libhal_device_get_property_strlist (hal_ctx, udi, key, &error);
+	} else {
+		allowed_options = libhal_device_get_property_strlist (hal_ctx, udi, "volume.mount.valid_options", &error);
+	}
 	if (dbus_error_is_set (&error)) {
 		unknown_error ("Cannot get volume.mount.valid_options");
 		dbus_error_free (&error);
@@ -689,7 +754,7 @@ handle_mount (LibHalContext *hal_ctx,
 						unknown_error ("option uid is malformed");
 					}
 #ifdef DEBUG
-					printf ("%s with uid %d\n", allow, uid);
+					printf ("%s with uid %u\n", allow, uid);
 #endif
 					wants_to_change_uid = TRUE;
 
@@ -722,6 +787,8 @@ handle_mount (LibHalContext *hal_ctx,
 		 * (since these doesn't contain uid/gid bits) 
 		 */
 		if (strcmp (libhal_volume_get_fstype (volume), "vfat") != 0 &&
+		    strcmp (libhal_volume_get_fstype (volume), "ntfs") != 0 &&
+		    strcmp (libhal_volume_get_fstype (volume), "ntfs-3g") != 0 &&
 		    strcmp (libhal_volume_get_fstype (volume), "iso9660") != 0 &&
 		    strcmp (libhal_volume_get_fstype (volume), "hfs") != 0 &&
 		    strcmp (libhal_volume_get_fstype (volume), "udf") != 0) {
@@ -729,21 +796,6 @@ handle_mount (LibHalContext *hal_ctx,
 		}
 	}
 
-
-	char *mount_option_commasep = NULL;
-	char *mount_do_fstype = "auto";
-
-	/* construct arguments to mount */
-	na = 0;
-	args[na++] = MOUNT;
-	if (strlen (mount_fstype) > 0) {
-		mount_do_fstype = (char *) map_fstype (mount_fstype);
-	} else if (volume == NULL) {
-		/* non-pollable drive; force auto */
-		mount_do_fstype = "auto";
-	} else if (libhal_volume_get_fstype (volume) != NULL && strlen (libhal_volume_get_fstype (volume)) > 0) {
-		mount_do_fstype = (char *) map_fstype (libhal_volume_get_fstype (volume));
-	}
 	args[na++] = MOUNT_TYPE_OPT;
 	args[na++] = mount_do_fstype;
 
@@ -788,15 +840,6 @@ handle_mount (LibHalContext *hal_ctx,
 #ifdef HAVE_POLKIT
         if (invoked_by_syscon_name != NULL) {
                 char *polkit_result;
-                char *action_params[] = {
-                        "fstype", "",
-                        "mount-point", "",
-                        "mount-options", "",
-                        NULL};
-
-                action_params[1] = mount_do_fstype;
-                action_params[3] = mount_dir;
-                action_params[5] = mount_option_commasep;
 
                 dbus_error_init (&error);
                 polkit_result = libhal_device_is_caller_privileged (hal_ctx,
